/* linux/arch/arm/mach-s5p64x0/include/mach/memory.h
 *
 * Copyright (c) 2009-2010 Samsung Electronics Co., Ltd.
 *		http://www.samsung.com
 *
 * S5P64X0 - Memory definitions
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#ifndef __ASM_ARCH_MEMORY_H
#define __ASM_ARCH_MEMORY_H __FILE__

#define PHYS_OFFSET		UL(0x20000000)
#define CONSISTENT_DMA_SIZE	SZ_8M

#endif /* __ASM_ARCH_MEMORY_H */
